/* WWW.LASERHARP.COM */
/*
* The Wireless Laser Harp Project
* TX/RX of laser sensor's data on 2.4GHz nRF24L01 transceiver 
* Based a very basic example of how to send data from one node to another
*
* Updated:Sept 2017 by www.laserharp.com Team
* Software by Greg Kendall
*
*/

#include <SPI.h>
#include "RF24.h"

/****************** User Config ***************************/
/***    Set this radio as radio number:
*       0 is for Sensor transmitter
*       1 is for sensor receiver
*       2 is for footswitch receiver
*/

// modify this to hardware switch
// 0 is for the gateway (receives from all pipes)
// 1 is for the sensor (transmits sensor data)
// 2 is for the footswitch (transmit footswitch position)

#define DIG_IN  2
#define DIG_OUT 2
#define LED 8
#define INT_PIN 0

#define DEV_GATEWAY           0
#define DEV_NODE_SENSOR       1
#define DEV_NODE_FOOTSWITCH   2

unsigned char bDeviceNumber = DEV_GATEWAY; 
//unsigned char bDeviceNumber = DEV_NODE_SENSOR; 

unsigned char bTest = 0X5a; // byte that we will send
unsigned char bRet ; // bye returned
bool bOldInputState = 0;
bool bNewInputState = 0;
uint8_t bRxPipe;

/* Hardware configuration: Set up nRF24L01 radio on SPI bus plus pins 7 & 8 */
RF24 radio(9,10); // CNS, CE
/**********************************************************/

byte addresses[][6] = {"0Gate","1Sens","2Ftsw"};

// Used to control whether this node is sending or receiving
bool role = 0;

void IRQService(void){
        EIMSK &= ~1;   // DISABLE INT0 

      EIFR &= 1;  // clear existing flags INT0 (done writing 1 to the flag)
      // got ultrasensor signal
      digitalWrite(LED,HIGH);  // LED
      // send a signal that ultrasensor detected a value
      radio.write( &bTest, sizeof(unsigned char) );
      EIFR &= 1;  // clear existing flags INT0 (I feel safe)
      EICRA |= 3;   // set wanted flags (RISING EDGE  interrupt)
      EIMSK |= 1;   // enable it

      // attachInterrupt( INT_PIN, IRQService, RISING); // enable this int again
       digitalWrite(LED,LOW);  // LED
}
void setup() {
  
    Serial.begin(115200);

    pinMode(LED,OUTPUT); 
    digitalWrite(LED,HIGH);  // LED
    pinMode(2,INPUT_PULLUP);
    pinMode(3,INPUT_PULLUP);
    pinMode(4,INPUT_PULLUP);
    pinMode(5,INPUT_PULLUP);

    radio.begin();

 // Set the PA Level low to prevent power supply related issues since this is a
 // getting_started sketch, and the likelihood of close proximity of the devices. RF24_PA_MAX is default.
  radio.setPALevel(RF24_PA_HIGH);
  
  // Open a writing and reading pipe on each radio, with opposite addresses
  // Up to 6 pipes can be open for reading at once. Open all the reading pipes, and then call startListening().
    radio.openWritingPipe(addresses[bDeviceNumber]);
    switch(bDeviceNumber){
        case 0: // Gateway connected to EVO
          pinMode(DIG_OUT,OUTPUT); // the digital input pin on device 1 here is the output (specular)
          digitalWrite(DIG_OUT,0);
          // DEVICE 0 RECEIVES FROM ALL PIPES
          radio.openReadingPipe(1,addresses[1]);
          radio.openReadingPipe(2,addresses[2]);
          // Start the radio listening for data
          radio.startListening();
        break;
        // other devices are TX only,but let's open pipes however. We just don't listen
        case 1: // Node attached to Ultrasensor
          radio.openReadingPipe(1,addresses[0]);
          radio.openReadingPipe(2,addresses[2]);
          pinMode(INT_PIN,INPUT_PULLUP);
          attachInterrupt( INT_PIN, IRQService, RISING);
          pinMode(DIG_IN,INPUT_PULLUP); // digital input connected to ultrasensor's dig out
        break;
        case 2: // Node attached to Footswitch
          radio.openReadingPipe(1,addresses[0]);
          radio.openReadingPipe(2,addresses[1]);
        break;
        default:
        break;
    }

}

void loop() {

    switch(bDeviceNumber){
        // device 0 Rx from all pipes, no Tx 
        case 0:
            if ( radio.available( &bRxPipe ) ){ 
                // received something   
                // bRxPipe contains the pipe from where we received something   
                radio.read( &bRet, sizeof(unsigned char) );  // read data from radio
                if (bRet == 0x5A){
                    // got ultrasensor signal
                    digitalWrite(DIG_OUT,HIGH);
                    digitalWrite(LED,HIGH);  // LED
                    delayMicroseconds(100); // helps EVO to detect the pulse
                    digitalWrite(DIG_OUT,LOW);
                    digitalWrite(LED,LOW);  // LED
                }
                else
                    digitalWrite(DIG_OUT,LOW); // do this if wrong data received

            }
            else{
                 digitalWrite(DIG_OUT,LOW); // do this if nothing available on radio
                 digitalWrite(LED,LOW);  // LED
            }

        break;
        // other devices are TX only,but let's open pipes however. We just don't listen
        case 1:
        /*
            bNewInputState = digitalRead(DIG_IN);
            if( bNewInputState & !bOldInputState ){
                bTest = 0x5A; // send a signal that ultrasensor detected a value
              //Serial.println(F("Now sending"));
              if (!radio.write( &bTest, sizeof(unsigned char) ))
                Serial.println(F("failed"));
            }
            bOldInputState = bNewInputState;
            // Delay 1ms
            delay(1);
            */
        break;
        case 2:
        break;
        default:
        break;
    }

} // Loop

